/**
 *  @file
 *
 *  @brief Create Idle Thread
 *  @ingroup ScoreThread
 */

/*
 *  COPYRIGHT (c) 1989-2011.
 *  On-Line Applications Research Corporation (OAR).
 *
 *  The license and distribution terms for this file may be
 *  found in the file LICENSE in this distribution or at
 *  http://www.rtems.org/license/LICENSE.
 */

#if HAVE_CONFIG_H
#include "config.h"
#endif

#include <rtems/score/threadimpl.h>
#include <rtems/score/schedulerimpl.h>
#include <rtems/score/stackimpl.h>
#include <rtems/config.h>

static void _Thread_Create_idle_for_cpu( Per_CPU_Control *cpu )
{
  Objects_Name    name;
  Thread_Control *idle;

  name.name_u32 = _Objects_Build_name( 'I', 'D', 'L', 'E' );

  /*
   *  The entire workspace is zeroed during its initialization.  Thus, all
   *  fields not explicitly assigned were explicitly zeroed by
   *  _Workspace_Initialization.
   */
  idle = _Thread_Internal_allocate();

  _Thread_Initialize(
    &_Thread_Internal_information,
    idle,
    _Scheduler_Get_by_CPU( cpu ),
    NULL,        /* allocate the stack */
    _Stack_Ensure_minimum( rtems_configuration_get_idle_task_stack_size() ),
    CPU_IDLE_TASK_IS_FP,
    PRIORITY_MAXIMUM,
    true,        /* preemptable */
    THREAD_CPU_BUDGET_ALGORITHM_NONE,
    NULL,        /* no budget algorithm callout */
    0,           /* all interrupts enabled */
    name
  );

  /*
   *  WARNING!!! This is necessary to "kick" start the system and
   *             MUST be done before _Thread_Start is invoked.
   */
  cpu->heir      =
  cpu->executing = idle;

  _Thread_Start(
    idle,
    THREAD_START_NUMERIC,
    rtems_configuration_get_idle_task(),
    NULL,
    0,
    cpu
  );
}

void _Thread_Create_idle( void )
{
  uint32_t cpu_count = _SMP_Get_processor_count();
  uint32_t cpu_index;

  for ( cpu_index = 0 ; cpu_index < cpu_count ; ++cpu_index ) {
    Per_CPU_Control *cpu = _Per_CPU_Get_by_index( cpu_index );

    if ( _Per_CPU_Is_processor_started( cpu ) ) {
      _Thread_Create_idle_for_cpu( cpu );
    }
  }
}
